#
#  Copyright 2004 VERITAS Software Corporation.
#  All rights reserved.
#
#  VERITAS, VERITAS SOFTWARE, the VERITAS logo and all other VERITAS
#  product names and slogans are trademarks or registered trademarks
#  of VERITAS Software Corporation in the USA and/or other countries.
#  Other product names and/or slogans mentioned herein may be trade-
#  marks or registered trademarks of their respective companies.
#  

eval 'exec $VCS_HOME/bin/perl5 -S $0 ${1+"$@"}'

if 0;

package HTCAgent;
use ArrayRep;
@ISA = ("ArrayRep");

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);

@EXPORT = qw(
);
$VERSION = '1.00';

my $vcs_home=$ENV{"VCS_HOME"} || "/opt/VRTSvcs";
use ag_i18n_inc;

%default = {
	resname => '',
	name	=> '',
	home	=> '',
	role	=> '',
	status  => '',
	split	=> '',
};

sub new {
	my($proto, $resname, $name, $home, $splittakeover) = @_;
	my $class = ref($proto) || $proto;
	my $self = { %default };
	$self->{'resname'} = $resname;
	$self->{'name'} = $name;
	$self->{'home'} = $home;
	$self->{'split'} = $splittakeover;
	$self->{'role'} = "";
	$self->{'start_time'} = $^T;
	bless($self, $class);
	return $self;
}

sub DESTROY {}

sub get_state {
	my $self = shift;
	$self->{'role'} = "";
	$self->{'status'} = "";
	my $cmd = $self->{'home'} . "/pairdisplay";
	my @query = split('\n', `$cmd -l -fc -CLI -g $self->{'name'}`);
	foreach my $q (@query) {
		if ($q =~ /Status/) {
			next;
		}
		my (@fields) = split('\s+', $q);
		if ($self->{'role'} eq "") {
			$self->{'role'} = $fields[8];
		} elsif ($self->{'role'} eq "P-VOL") {
			if ($fields[8] ne "P-VOL") {
				$self->{'role'} = "Invalid";
				last;
			}
		} elsif ($self->{'role'} eq "S-VOL") {
			if ($fields[8] ne "S-VOL") {
				$self->{'role'} = "Invalid";
				last;
			}
		} else {
			$self->{'role'} = "Invalid";
		}
		if ($self->{'status'} eq "") {
			$self->{'status'} = $fields[9];
			next;
		}
		if (($fields[9] eq "COPY") || ($fields[9] eq "PSUE") || ($fields[9] eq "SSUS")) {
			$self->{'status'} = $fields[9];
		}
	}
	VCSAG_LOG_MSG("N", "state of $self->{'name'} is $self->{'status'}; role is $self->{'role'}", 1, "$self->{'name'}", "$self->{'status'}", "$self->{'role'}"); 
}


sub failover_required {
	my ($self) = shift;
	if ($self->{'role'} eq "Invalid") {
		return 2;
	}
	# S-VOL is writable in SSWS state; merely using HTC terminology here
	if (($self->{'role'} eq "S-VOL") && ($self->{'status'} eq "SSUS")) {
		return 3;
	}

	if (($self->{'role'} eq "S-VOL") && ($self->{'status'} eq "PAIR")) {

	        VCSAG_LOG_MSG("N", "checking for replication link failure", 3);

		my $cmd = $self->{'home'} . "/pairdisplay";
		my @query = split('\n', `$cmd -fc -CLI -g $self->{'name'}`);
		foreach my $q (@query) {
			if ($q =~ /Status/) {
				next;
			}
			my (@fields) = split('\s+', $q);

			if ($fields[8] ne "P-VOL") {
					next;
			}

			if ($fields[9] eq "PSUE") {
				return 4;
			}
		}
		return 5;
	}

	if (($self->{'role'} eq "S-VOL") && ($self->{'status'} ne "SSWS")) {
		return 1;
	}
	return 0;
}

sub wait_for_sync {
	my ($self) = shift;
	if (($self->{'role'} eq "S-VOL") && ($self->{'status'} eq "COPY")) {
		VCSAG_LOG_MSG("N", "COPY status detected; entry point will wait until all volumes in PAIR status or entry point times out", 2);
		my ($oto) = split('\n', `$vcs_home/bin/hatype -value HTC OnlineTimeout`);
		my $toval = $oto - 30;
		my $cmdtxt = $self->{'home'} . "/pairevtwait -g " . $self->{'name'} . " -s pair -t " . $toval;
	        VCSAG_LOG_MSG("N", "issuing pairevtwait of device group $self->{'name'} with command: $cmdtxt", 3, "$self->{'name'}", "$cmdtxt");
		my $cmd = $self->{'home'} . "/pairevtwait -g " . $self->{'name'} . " -s pair -t $toval";
		my $ret = system("$cmd > /dev/null");
		if ($ret) {
			VCSAG_LOG_MSG("E", "COPY did not complete before entry point timeout", 4);
			exit(0);
		}
		#TBD: to SSWS from SSUS
	}
	return 0;
}

sub failover {
	my ($self) = @_;
	my ($oto) = split('\n', `$vcs_home/bin/hatype -value HTC OnlineTimeout`);
	my $toval = $oto - 30;
	my $cmdtxt = $self->{'home'} . "/horctakeover -t " . $toval . " -g " . $self->{'name'} . " 2>&1";
        VCSAG_LOG_MSG("N", "issuing horctakover of device group $self->{'name'} with command: $cmdtxt", 5, "$self->{'name'}", "$cmdtxt");
	my $ret = system("$self->{'home'}/horctakeover -t $toval -g $self->{'name'} 2>&1");
	$ret = $ret / 256;
        VCSAG_LOG_MSG("N", "horctakeover was issued; return code is $ret", 6, "$ret");
	if ($ret <= 5) {
		$self->create_lockfile();
		return 0;
	}
	$self->get_state();
	if ($self->{'role'} eq "P-VOL") {
		$self->create_lockfile();
		return 0;
	}
	if ($self->{'role'} eq "S-VOL") {
		my $cur = time();
		my $maxloop = ($cur - $self->{'start_time'}) / 5;
		my $numloop = 0;
		while ((($self->{'status'} eq "COPY") || ($self->{'status'} eq "PAIR")) && ($numloop < $maxloop)) {
			VCSAG_LOG_MSG("N", "waiting for COPY/PAIR to transition...", 7);
			sleep(5);
			$numloop = $numloop + 1;
			$self->get_state();
		}
		if (($self->{'status'} eq "SSWS") || ($self->{'status'} eq "SSUS")) {
			if ($ret == 225) {
				my $cmdtxt = $self->{'home'} . "/pairresync -g " . $self->{'name'} . " -swaps";
			        VCSAG_LOG_MSG("N", "issuing optional pairresync -swaps of device group $self->{'name'} in case array is up but RAID manager down with command: $cmdtxt", 3, "$self->{'name'}", "$cmdtxt");
				my $cmd = $self->{'home'} . "/pairresync -g " . $self->{'name'} . " -swaps";
				my $ret = system("$cmd 2>&1");
			}
			$self->create_lockfile();
			return 0;
		}
	}
	VCSAG_LOG_MSG("W", "horctakeover failed; error code is $ret", 8, "$ret");
	exit(0);
}

sub clearfd {
	my ($self) = @_;
	my (@lines) = split('\n', `$vcs_home/bin/hares -list TargetResName=$self->{'resname'} Type=HTCSnap`);
	if ($#lines == -1) {
		return 0;
	}
	my (@nodefields) = split('\s+', `$vcs_home/bin/hasys -nodeid`);
	my ($fdres, $sys) = split('\s+', $lines[$#lines]);
	my ($grp) = split('\n', `$vcs_home/bin/hares -value $fdres Group`);
	my (@flines) = split('\n', `$vcs_home/bin/hares -list Type=HTCSnap Group=$grp`);
	my $prevres = "";
	foreach my $fline (@flines) {
		my ($fdres, $sys) = split('\s+', $fline);
		if (!$prevres || ($prevres ne $fdres)) {
			$prevres = $fdres;
			my $ret = system("$vcs_home/bin/hares -action $fdres clearvm -sys $nodefields[5]");
			if ($ret) {
				VCSAG_LOG_MSG("E", "cannot clear VM headers from fire drill", 23);
				exit(0);
			}
		} else {
			next;
		}
	}
	return 0;
}

sub linkdown {
	my ($self) = @_;
	if (($self->{'role'} eq "P-VOL") && ($self->{'status'} eq "PSUE")) {
		return 1;
	}
	return 0;
}

sub linkrestored {
	my ($self) = @_;
	if (($self->{'role'} eq "P-VOL") && ($self->{'status'} ne "PSUE")) {
		return 1;
	}
	return 0;
}

sub resynchronize {
	my ($self) = @_;
	my $cmdtxt = $self->{'home'} . "/pairresync -g " . $self->{'name'};
	my $cmd = $self->{'home'} . "/pairresync -g " . $self->{'name'};
	my $ret = system("$cmd > /dev/null 2>&1");
	if (!$ret) {
	        VCSAG_LOG_MSG("N", "automatic link resynchronization of group $self->{'name'} succeeded with command with command: $cmdtxt", 24, "$self->{'name'}", "$cmdtxt");
	}
	return 0;
}

sub freezetarget {
	my ($self) = @_;
	my ($val) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} TargetFrozen`);
	if ($val) {
		return 0;
	}
	my (@cluslist) = split('\n', `$vcs_home/bin/haclus -list`);
	if ($#cluslist < 1) {
		return;
	}
	my ($grp) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} Group`);
	my (@lines) = split('\n', `$vcs_home/bin/hagrp -value $grp TFrozen -clus $cluslist[1]`);
	if ($#lines < 1) {
		return;
	}
	my (@fields) = split('\s+', $lines[1]);
	if ($fields[3] eq "1") {
		return;
	}
	my $ret = system("$vcs_home/bin/hagrp -freeze $grp -clus $cluslist[1]");
	if (!$ret) {
	        VCSAG_LOG_MSG("N", "service group $grp in cluster $cluslist[1] was frozen because the replication link has failed; group will be unfrozen when data is resynchronized", 25, "$grp", "$cluslist[1]");
		my $val = $grp . ":" . $cluslist[1];
		system("$vcs_home/bin/hares -modify $self->{'resname'} TargetFrozen $val");
	}
	return 0;
}

sub unfreezetarget {
	my ($self) = @_;
	my ($val) = split('\n', `$vcs_home/bin/hares -value $self->{'resname'} TargetFrozen`);
	if (!$val) {
		return 0;
	}
	my ($grp, $clus) = split(':', $val);
	my $ret = system("$vcs_home/bin/hagrp -unfreeze $grp -clus $clus");
	if (!$ret) {
	        VCSAG_LOG_MSG("N", "service group $grp in cluster $clus was unfrozen because the replication link has been resynchronized", 26, "$grp", "$clus");
		system("$vcs_home/bin/hares -modify $self->{'resname'} TargetFrozen \"\"");
	}
	return 0;
}


1;

